@@ -11,11 +11,10 @@ import torch as _torch
 from torch.utils.data import DataLoader as _DataLoader, Dataset as _Dataset
 from torch.utils.data._utils.collate import default_collate as _default_collate
 
+from easytorch.core import metrics as _base_metrics
 from easytorch.core import utils as _utils
 from easytorch.utils import logutils as _log_utils
 from easytorch.utils.datautils import _init_kfolds
-from easytorch.core import metrics as _base_metrics
-import warnings as _warn
 
 _sep = _os.sep
 
@@ -34,6 +33,22 @@ class ETTrainer:
         self.device = _ODict()
         self.optimizer = _ODict()
 
+        """
+        Default log holders for each of the k-fold. can override _reset_fold_cache to make it different.
+        """
+        self.cache['training_log'] = ['LOSS,Accuracy']
+        self.cache['validation_log'] = ['LOSS,Accuracy']
+        self.cache['test_score'] = ['LOSS,Accuracy']
+
+        """
+        Default global score holder for each datasets.
+        Save the latest time(maximize current time.). One can also maximize/minimize any other score from
+            easytorch.nn.metrics.ETMetrics() class by overriding _reset_dataset_cache.
+        """
+        self.cache['global_test_score'] = []
+        self.cache['monitor_metric'] = 'time'
+        self.cache['metric_direction'] = 'maximize'
+
     def init_nn(self):
         r"""
         Call to user implementation of:
@@ -133,7 +148,6 @@ class ETTrainer:
         accuracy, precision in the method named metrics() to be able to track and plot it.
         Example: easytorch.utils.measurements.Pr11a() will work with precision, recall, F1, Accuracy, IOU scores.
         """
-        _warn.warn('Base easytoch.core.metrics.ETMetrics() initialized. If it is on purpose, ignore this warning,')
         return _base_metrics.ETMetrics()
 
     def new_averages(self):
@@ -141,7 +155,6 @@ class ETTrainer:
         Should supply an implementation of easytorch.core.metrics.ETAverages() that can keep track of multiple averages.
         For example, multiple loss, or any other values.
         """
-        _warn.warn('Base easytoch.core.metrics.ETAverages() initialized. If it is on purpose, ignore this warning,')
         return _base_metrics.ETAverages(num_averages=1)
 
     def check_previous_logs(self):
@@ -194,7 +207,7 @@ class ETTrainer:
             # Maximize(eg. F1/Accuracy) OR Minimize(eg. MSE, RMSE, COSINE)
             self.cache['metric_direction'] = 'maximize'
         """
-        raise NotImplementedError('Must be implemented in child class.')
+        pass
 
     def reset_fold_cache(self):
         r"""
@@ -216,7 +229,7 @@ class ETTrainer:
         self.cache['validation_log'] = ['Loss,Precision,Recall,F1,Accuracy']
         self.cache['test_score'] = ['Split,Precision,Recall,F1,Accuracy']
         """
-        raise NotImplementedError('Must be implemented in child class.')
+        pass
 
     def save_if_better(self, epoch, metrics):
         r"""
@@ -258,7 +271,6 @@ class ETTrainer:
             -we need to keep track of loss
             -we need to keep track of metrics
         """
-        _warn.warn('Base iteration initialized. If it is on purpose, ignore this warning.')
         return {'metrics': _base_metrics.ETMetrics(), 'averages': _base_metrics.ETAverages(num_averages=1)}
 
     def save_predictions(self, dataset, its):
@@ -292,7 +304,11 @@ class ETTrainer:
                 its = []
                 metrics = self.new_metrics()
                 for i, batch in enumerate(loader):
+
                     it = self.iteration(batch)
+                    if not it.get('metrics'):
+                        it['metrics'] = _base_metrics.ETMetrics()
+
                     metrics.accumulate(it['metrics'])
                     eval_loss.accumulate(it['averages'])
                     if save_pred:
@@ -328,7 +344,7 @@ class ETTrainer:
         """
         pass
 
-    def _on_iteration_end(self, i, it):
+    def _on_iteration_end(self, i, ep, it):
         r"""
         Any logic to run after an iteration ends.
         """
@@ -361,6 +377,9 @@ class ETTrainer:
             for i, batch in enumerate(train_loader, 1):
 
                 it = self.training_iteration(batch)
+                if not it.get('metrics'):
+                    it['metrics'] = _base_metrics.ETMetrics()
+
                 ep_loss.accumulate(it['averages'])
                 ep_metrics.accumulate(it['metrics'])
                 _loss.accumulate(it['averages'])
@@ -370,12 +389,14 @@ class ETTrainer:
                           f"{_loss.get()},{_metrics.get()}")
                     _metrics.reset()
                     _loss.reset()
-                self._on_iteration_end(i, it)
+                self._on_iteration_end(i, ep, it)
 
             self.cache['training_log'].append([*ep_loss.get(), *ep_metrics.get()])
+
             val_loss, val_metric = self.evaluation(split_key='validation', dataset_list=[val_dataset])
             self.save_if_better(ep, val_metric)
             self.cache['validation_log'].append([*val_loss.get(), *val_metric.get()])
+
             _log_utils.plot_progress(self.cache, experiment_id=self.cache['experiment_id'],
                                      plot_keys=['training_log', 'validation_log'])
             self._on_epoch_end(ep, ep_loss, ep_metrics, val_loss, val_metric)
